package com.jkcredit.invoice.service.lowerService.impl;

import cn.com.taiji.sdk.model.comm.protocol.tts.invoice.server.*;
import cn.com.taiji.sdk.model.comm.protocol.tts.trade.service.CardTradeModel;
import cn.com.taiji.sdk.model.comm.protocol.tts.trade.service.CardTradeRequest;
import cn.com.taiji.sdk.model.comm.protocol.tts.trade.service.CardTradeResponse;
import com.jkcredit.invoice.mapper.Binvoce.SelfCarApplMapper;
import com.jkcredit.invoice.mapper.Binvoce.SelfCarInvoiceMapper;
import com.jkcredit.invoice.mapper.calculateInfor.SelfCarCalculateInforMapper;
import com.jkcredit.invoice.mapper.customer.CustomerCarRecMapper;
import com.jkcredit.invoice.mapper.customer.CustomerMapper;
import com.jkcredit.invoice.mapper.customer.CustomerRecMapper;
import com.jkcredit.invoice.mapper.waybill.SellCarTradeMapper;
import com.jkcredit.invoice.model.entity.Calculate.SelfCarCalculateInfor;
import com.jkcredit.invoice.model.entity.customer.Customer;
import com.jkcredit.invoice.model.entity.customer.CustomerCarRec;
import com.jkcredit.invoice.model.entity.customer.CustomerRec;
import com.jkcredit.invoice.model.entity.invoice.SelfCarAppl;
import com.jkcredit.invoice.model.entity.invoice.SelfCarInvoice;
import com.jkcredit.invoice.model.entity.waybill.SelfCarTrade;
import com.jkcredit.invoice.service.lowerService.SelfCarServiceL;
import com.jkcredit.invoice.service.lowerService.vo.InvoiceApplVo;
import com.jkcredit.invoice.service.lowerService.vo.SelfCarDueQueryVo;
import com.jkcredit.invoice.service.lowerService.vo.TradeRequestVo;
import com.jkcredit.invoice.service.upService.SelfCarInterface;
import com.jkcredit.invoice.util.DateUtil;
import com.jkcredit.invoice.util.RespR;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
@Service
public class SelfCarServiceLImpl implements SelfCarServiceL {
    @Autowired
    SelfCarInterface selfCarInterface;
    @Autowired
    CustomerCarRecMapper customerCarRecMapper;
    @Autowired
    CustomerMapper customerMapper;
    @Autowired
    CustomerRecMapper customerRecMapper;
    @Autowired
    SellCarTradeMapper sellCarTradeMapper;
    @Autowired
    SelfCarApplMapper selfCarApplMapper;
    @Autowired
    SelfCarInvoiceMapper selfCarInvoiceMapper;
    @Autowired
    SelfCarCalculateInforMapper calculateInforMapper;
    @Override
    public RespR<List<SelfCarTrade>> getTradeList(TradeRequestVo tradeRequestVo) {
        Customer customer = customerMapper.selectByCustomerName(tradeRequestVo.getCustomerName());
        if(customer == null || customer.getAccstatus().intValue() == 1){
            return new RespR(false,"客户已经停用");
        }
        if(DateUtil.getDistanceDays(tradeRequestVo.getEndTime(),tradeRequestVo.getStartTime())>30){
            return new RespR(false,"不能查询大于90天的交易");
        }
        CustomerRec customerRec = new CustomerRec();
        customerRec.setCustomerName(tradeRequestVo.getCustomerName());
        customerRec.setCompanyName(tradeRequestVo.getCompanyName());
        customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRec);
        if(customerRec == null || customerRec.getRecStatus().intValue() ==1 ){
            return new RespR(false,"客户未备案");
        }
        CustomerCarRec customerCarRec = customerCarRecMapper.selectByETC(tradeRequestVo.getEtcId());
        if(customerCarRec == null || customerCarRec.getRecStatus() != 1){
            return new RespR(false,"该etc卡未绑定或绑定类型错误");
        }
        CardTradeRequest cardTradeRequest = new CardTradeRequest();
        cardTradeRequest.setCardId(tradeRequestVo.getEtcId());
        cardTradeRequest.setCompanyNum(customerRec.getCompanyNum());
        cardTradeRequest.setStartExTime(tradeRequestVo.getStartTime());
        cardTradeRequest.setEndExTime(tradeRequestVo.getEndTime());
        cardTradeRequest.setTradeStatus(tradeRequestVo.getTradeStatus());
        RespR<CardTradeResponse> resp = selfCarInterface.cardTrade(cardTradeRequest);
        if(resp.getCode() == 1){
            return new RespR(false,"请求失败："+resp.getMsg());
        }else{
            CardTradeResponse cardTradeResponse = resp.getData();
            List<CardTradeModel> result  = cardTradeResponse.getResult();
            //对不存在的tradeId入表
            if(result!=null && result.size()>0){
                for(CardTradeModel cardTradeModel:result){
                    SelfCarTrade selfCarTrade =  sellCarTradeMapper.selectByTradeId(cardTradeModel.getTradeId());
                    if(selfCarTrade == null){
                        selfCarTrade = new SelfCarTrade();
                        selfCarTrade.setCustomId(customerRec.getCustomerName());
                        selfCarTrade.setCompanyNum(customerRec.getCompanyNum());
                        selfCarTrade.setCardId(cardTradeModel.getCardId());
                        selfCarTrade.setTradeId(cardTradeModel.getTradeId());
                        selfCarTrade.setAclTime(cardTradeModel.getExTime());
                        selfCarTrade.setFee(cardTradeModel.getFee());
                        selfCarTrade.setStatus(0);
                        sellCarTradeMapper.insert(selfCarTrade);
                        //计费
                        SelfCarCalculateInfor carCalculateInfor = new SelfCarCalculateInfor();
                        carCalculateInfor.setCustomId(customerRec.getCustomerName());
                        carCalculateInfor.setCompanyNum(customerRec.getCompanyName());
                        carCalculateInfor.setCalTime(StringUtils.substring(cardTradeModel.getExTime(),0,6));
                        carCalculateInfor.setFee(customer.getFee());
                        carCalculateInfor.setEtcNum(cardTradeModel.getCardId());
                        if(calculateInforMapper.selectByEtcAndTime(carCalculateInfor) == null){
                            calculateInforMapper.insert(carCalculateInfor);
                        }

                    }
                };
            }
            return new RespR(result);
        }
    }

    @Override
    @Transactional
    public RespR<Boolean> applInvoice(InvoiceApplVo invoiceApplVo) {
        Customer customer = customerMapper.selectByCustomerName(invoiceApplVo.getCustomerName());
        if(customer == null || customer.getAccstatus().intValue() == 1){
            return new RespR(false,"客户已经停用");
        }
        CustomerRec customerRec = new CustomerRec();
        customerRec.setCustomerName(invoiceApplVo.getCustomerName());
        customerRec.setCompanyName(invoiceApplVo.getCompanyName());
        customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRec);
        if(customerRec == null || customerRec.getRecStatus().intValue() ==1 ){
            return new RespR(false,"客户未备案");
        }
        CustomerCarRec customerCarRec = customerCarRecMapper.selectByETC(invoiceApplVo.getCardId());
        if(customerCarRec == null || customerCarRec.getRecStatus() != 1){
            return new RespR(false,"该etc卡未绑定或绑定类型错误");
        }
        B2BInvoiceApplyRequest b2BInvoiceApplyRequest = new B2BInvoiceApplyRequest();
        b2BInvoiceApplyRequest.setCardId(invoiceApplVo.getCardId());
        b2BInvoiceApplyRequest.setCompanyNum(customerRec.getCompanyNum());
        List<B2BInvoiceApplyModel> b2BInvoiceApplyModels = new ArrayList<>();
        List<String> tradeIds = invoiceApplVo.getTradeIds();
        if(tradeIds == null || tradeIds.size()<=0){
            return new RespR<>(false,"没有交易Id");
        }
        for(String tradeId:tradeIds){
            B2BInvoiceApplyModel b2BInvoiceApplyModel = new B2BInvoiceApplyModel();
            b2BInvoiceApplyModel.setTradeId(tradeId);
            b2BInvoiceApplyModels.add(b2BInvoiceApplyModel);
        }
        b2BInvoiceApplyRequest.setTradeIdModel(b2BInvoiceApplyModels);
        RespR<B2BInvoiceApplyResponse> respR = selfCarInterface.b2BInvoiceApply(b2BInvoiceApplyRequest);
        if(respR.getCode()==1){
            return new RespR<>(false,"申请失败");
        }else{
            B2BInvoiceApplyResponse b2BInvoiceApplyResponse = respR.getData();
            SelfCarAppl selfCarAppl = new SelfCarAppl();
            selfCarAppl.setApplId(b2BInvoiceApplyResponse.getApplyId());
            selfCarAppl.setApplTime(b2BInvoiceApplyResponse.getReceiveTime());
            selfCarAppl.setCompanyNum(b2BInvoiceApplyRequest.getCompanyNum());
            selfCarAppl.setCustomId(customer.getCustomerName());
            selfCarAppl.setCardId(invoiceApplVo.getCardId());
            selfCarAppl.setInvoiceAmount(b2BInvoiceApplyResponse.getInvoiceAmount());
            selfCarAppl.setInvoiceCount(b2BInvoiceApplyResponse.getInvoiceCount());
            selfCarAppl.setInvoiceTaxAmount(b2BInvoiceApplyResponse.getInvoiceTaxAmount());
            selfCarApplMapper.insert(selfCarAppl);
            for(String tradeId:tradeIds){
               SelfCarTrade selfCarTrade = sellCarTradeMapper.selectByTradeId(tradeId);
               selfCarTrade.setApplId(b2BInvoiceApplyResponse.getApplyId());
               selfCarTrade.setStatus(1);
               sellCarTradeMapper.updateByPrimaryKey(selfCarTrade);
            }
        }
        return new RespR<>(true);
    }

    @Override
    public RespR<List<SelfCarInvoice>> getSelfCarInvoicesByApplyIds(SelfCarAppl selfCarAppl) {
        //TODO 查找是否已经开过票 如果开过就直接返回，查看所有返回的时机
       // selfCarApplMapper

        B2BInvoiceQueryByApplyRequest b2BInvoiceQueryRequest = new B2BInvoiceQueryByApplyRequest();
        b2BInvoiceQueryRequest.setCardId(selfCarAppl.getCardId());
        b2BInvoiceQueryRequest.setCompanyNum(selfCarAppl.getCompanyNum());
        b2BInvoiceQueryRequest.setApplyId(selfCarAppl.getApplId());
        RespR<B2BInvoiceQueryByApplyResponse> responseRespR = selfCarInterface.b2bContractQuery(b2BInvoiceQueryRequest);
        if(responseRespR.getCode() == 1){
            return new RespR(false,responseRespR.getMsg());
        }else{
            B2BInvoiceQueryByApplyResponse response = responseRespR.getData();
            List<B2BInvoiceQueryByApplyModel> b2BInvoiceQueryByApplyModels = response.getResult();
            Set<String> tradeIds = new HashSet<>();
            for(B2BInvoiceQueryByApplyModel b2BInvoiceQueryByApplyModel:b2BInvoiceQueryByApplyModels){
                SelfCarInvoice selfCarInvoice = new SelfCarInvoice();
                BeanUtils.copyProperties(b2BInvoiceQueryByApplyModel,selfCarInvoice);
                tradeIds.add(selfCarInvoice.getTradeId());
                selfCarInvoiceMapper.insert(selfCarInvoice);
            }
            for(String tradeId:tradeIds){
                //更新tradeId为开票完成
                SelfCarTrade selfCarTrade = sellCarTradeMapper.selectByTradeId(tradeId);
                selfCarTrade.setStatus(2);
                sellCarTradeMapper.updateByPrimaryKey(selfCarTrade);

            }

            //返回相应的发票
            return new RespR(b2BInvoiceQueryByApplyModels);
        }
    }

    @Override
    public RespR<List<SelfCarInvoice>> getSelfCarInvoicesByTime(SelfCarDueQueryVo selfCarDueQueryVo) {
        Customer customer = customerMapper.selectByCustomerName(selfCarDueQueryVo.getCustomername());
        if(customer == null || customer.getAccstatus().intValue() == 1){
            return new RespR(false,"客户已经停用");
        }
        CustomerRec customerRec = new CustomerRec();
        customerRec.setCustomerName(selfCarDueQueryVo.getCustomername());
        customerRec.setCompanyName(selfCarDueQueryVo.getCompanyName());
        customerRec = customerRecMapper.selectByCustomerNameAndCompany(customerRec);
        if(customerRec == null || customerRec.getRecStatus().intValue() !=1 ){
            return new RespR(false,"客户未备案");
        }
        CustomerCarRec customerCarRec = customerCarRecMapper.selectByETC(selfCarDueQueryVo.getCardId());
        if(customerCarRec == null  || customerCarRec.getRecStatus() != 1){
            return new RespR(false,"该etc卡未绑定或绑定类型错误");
        }
        B2BInvoiceQueryRequest b2BInvoiceQueryRequest = new B2BInvoiceQueryRequest();
        b2BInvoiceQueryRequest.setCompanyNum(customerRec.getCompanyNum());
        b2BInvoiceQueryRequest.setCardId(selfCarDueQueryVo.getCardId());
        b2BInvoiceQueryRequest.setEndInvoiceMakeTime(selfCarDueQueryVo.getStartTime());
        b2BInvoiceQueryRequest.setEndInvoiceMakeTime(selfCarDueQueryVo.getEndTime());

        RespR<B2BInvoiceQueryResponse> responseRespR = selfCarInterface.b2BInvoiceQuery(b2BInvoiceQueryRequest);
        if(responseRespR.getCode() == 1){
            return new RespR(false,responseRespR.getMsg());
        }else{
            B2BInvoiceQueryResponse b2BInvoiceQueryResponse = responseRespR.getData();
            List<B2BInvoiceListModel> b2BInvoiceListModels = b2BInvoiceQueryResponse.getResult();
            List<SelfCarInvoice> selfCarInvoices = new ArrayList<>();
            if(b2BInvoiceListModels!=null && b2BInvoiceListModels.size()>0){
                for(B2BInvoiceListModel b2BInvoiceListModel:b2BInvoiceListModels){
                    SelfCarInvoice selfCarInvoice = new SelfCarInvoice();
                    BeanUtils.copyProperties(b2BInvoiceListModel,selfCarInvoice);
                    selfCarInvoices.add(selfCarInvoice);
                }
            }
            return new RespR(selfCarInvoices);
        }
    }

}
